@@ -1,3 +1,8 @@
+import os
+import shutil
+
+import numpy as np
+
 from guide_bot.parameters import instrument_parameters as ipars
 
 
@@ -129,6 +134,9 @@ class Element:
         self.start_point_parameter = None
         self.next_start_point_parameter = None
 
+        self.reference_component_name = None
+        self.end_component_name = name + "_end_point"
+
         self.owner = None
 
     def set_owner(self, owner):
@@ -276,7 +284,7 @@ class GuideElement(Element):
             else:
                 constants = []
             par = ipars.DependentInstrumentParameter(variable_name, dependent_on=kwargs["dependent_on"],
-                                               func=kwargs["func"], constants=constants)
+                                                     func=kwargs["func"], constants=constants)
         elif len(args) == 2:
             par = ipars.FixedInstrumentParameter(variable_name, args[1])
         elif len(args) == 3:
@@ -289,7 +297,7 @@ class GuideElement(Element):
     def set_end_dimensions(self, end_width, end_height):
         """
         Sets the end_width / height attributes, these have a type derived
-        from InstrumentParamter.
+        from InstrumentParameter.
         """
 
         self.end_width = end_width
@@ -312,7 +320,7 @@ class GuideElement(Element):
         next_cb_name = self.next_start_point_parameter.name
         length_var_name = self.name + "_length"
         self.current_instrument.add_declare_var("double", length_var_name)
-        self.current_instrument.append_initialize(length_var_name + " = " + next_cb_name + " - " + previous_cb_name + "-1E-6;")
+        self.current_instrument.append_initialize(length_var_name + " = " + next_cb_name + " - " + previous_cb_name + ";")
         
         self.length_parameter = length_var_name
     
@@ -353,19 +361,91 @@ class GuideElement(Element):
 
         self.write_to_log_base(file)
 
+    def copy_component(self, name, destination):
+        this_file_path = os.path.abspath(__file__)
+        this_directory = os.path.split(this_file_path)[0]
+        component_path = os.path.join(this_directory, "..", "..", "McStas_components", name)
+        destination_path = os.path.join(destination, name)
+
+        shutil.copyfile(component_path, destination_path)
+
+    def copy_components(self, destination):
+        return []
+
     def __repr__(self):
         """
         Provides a basic description of the guide element
         """
+
+        name_length = 12
         string = "GuideElement: "
 
         string += self.name + " : \n"
-        string += self.length.__repr__() + "\n"
-        string += self.start_point.__repr__() + "\n"
-        string += self.start_width.__repr__() + "\n"
-        string += self.start_height.__repr__() + "\n"
+        string += "length:".ljust(name_length) + self.length.__repr__() + "\n"
+        string += "start:".ljust(name_length) + self.start_point.__repr__() + "\n"
+        string += "s width:".ljust(name_length) + self.start_width.__repr__() + "\n"
+        string += "s height:".ljust(name_length) + self.start_height.__repr__() + "\n"
+        if self.end_width is not None:
+            string += "e width:".ljust(name_length) + self.end_width.__repr__() + "\n"
+        else:
+            string += "no end_width set\n"
+        if self.end_height is not None:
+            string += "e height:".ljust(name_length) + self.end_height.__repr__() + "\n"
+        else:
+            string += "no end_height set\n"
 
-        string += self.start_point_parameter.__repr__() + "\n"
-        string += self.next_start_point_parameter.__repr__()
+        string += "start par:".ljust(name_length) + self.start_point_parameter.__repr__() + "\n"
+        string += "n start p:".ljust(name_length) + self.next_start_point_parameter.__repr__()
 
         return string
+
+
+class PositionAndRotation:
+    def __init__(self, position, rotation):
+        self.position = position
+        self.rotation = rotation
+
+    def width_points(self, width):
+
+        x_vector = np.array([0.5, 0, 0])
+        x_vector = self.rotation.apply(x_vector)
+
+        m_position = self.position - width * x_vector
+        p_position = self.position + width * x_vector
+
+        return m_position, p_position
+
+    def height_points(self, height):
+
+        y_vector = np.array([0, 0.5, 0])
+        y_vector = self.rotation.apply(y_vector)
+
+        m_position = self.position - height * y_vector
+        p_position = self.position + height * y_vector
+
+        return m_position, p_position
+
+    def get_points(self, distance, horizontal):
+        """
+        Provides 3D points for plotting
+
+        Takes distance between points and wether it is in the
+        horizontal direction or vertical (True / False).
+
+        distance: float
+            distance between points
+
+        horizontal: bool
+            are they spaced horizontally or vertically
+        """
+        if horizontal:
+            return self.width_points(distance)
+        else:
+            return self.height_points(distance)
+
+    def __repr__(self):
+        string = "PositionAndRotation\n"
+        string += " Position: " + str(self.position) + "\n"
+        string += " Rotation: \n" + str(self.rotation.as_matrix()) + "\n"
+
+        return string
\ No newline at end of file
