@@ -9,12 +9,15 @@ from guide_bot.logging.log_plotter_helpers import LogParameter
 from guide_bot.logging.log_plotter_helpers import GuideElementLog
 from guide_bot.logging.log_plotter_helpers import sort_permutation
 from guide_bot.logging.log_plotter_helpers import extract_parameters
+from guide_bot.base_elements.guide_elements import PositionAndRotation
 
-
+import guide_bot.elements.Element_straight as Gap
 import guide_bot.elements.Element_straight as Straight
 import guide_bot.elements.Element_elliptic as Elliptic
+import guide_bot.elements.Element_curved as Curved
 import guide_bot.elements.Element_slit as Slit
 
+from scipy.spatial.transform import Rotation as R
 
 class LogPlotter:
     def __init__(self, filename):
@@ -173,6 +176,104 @@ class LogPlotter:
         else:
             ax.set_ylabel("Vertical [m]")
 
+    def plot_guide_center_line_ax(self, ax, horizontal=True, data_line=None):
+
+        # If no data is given, plot best configuration
+        if data_line is None:
+            data_line = self.data_fom_sorted[-1, :]
+
+        # start position and rotation
+        start_rot = R.from_euler("z", 0)
+        pos_and_rot = PositionAndRotation(np.array([0, 0, 0]), start_rot)
+
+        # plot source
+        par_dict = extract_parameters(self.guide_element_logs[0], self.legend, data_line)
+        if horizontal:
+            if "start_width" in par_dict:
+                moderator_width = par_dict["start_width"]
+                m_point, p_point = pos_and_rot.width_points(moderator_width)
+                ax.plot([m_point[2], p_point[2]], [m_point[0], p_point[0]], color="dimgrey")
+
+        else:
+            if "start_height" in par_dict:
+                moderator_height = par_dict["start_height"]
+                m_point, p_point = pos_and_rot.width_points(moderator_height)
+                ax.plot([m_point[1], p_point[1]], [m_point[0], p_point[0]], color="dimgrey")
+
+        color = "k"
+        for guide_element in self.guide_element_logs:
+
+            if color == "k":
+                color = "b"
+            elif color == "b":
+                color = "k"
+
+            par_dict = extract_parameters(guide_element, self.legend, data_line)
+
+            if guide_element.element_type == "Slit":
+                center_function = Slit.center_line
+
+            elif guide_element.element_type == "Gap":
+                center_function = Gap.center_line
+
+            elif guide_element.element_type == "Straight":
+                center_function = Straight.center_line
+                dim_function = Straight.guide_dimensions
+                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 2, horizontal, color)
+
+            elif guide_element.element_type == "Curved":
+                center_function = Curved.center_line
+                dim_function = Curved.guide_dimensions
+                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 20, horizontal, color)
+
+            elif guide_element.element_type == "Elliptic":
+                center_function = Elliptic.center_line
+                dim_function = Elliptic.guide_dimensions
+                self.plot_element(ax, pos_and_rot, center_function, dim_function, par_dict, 30, horizontal, color)
+
+            else:
+                raise RuntimeError("Unknown type in plotter!")
+
+            pos_and_rot = center_function(pos_and_rot, par_dict, 1.0)
+
+        # plot sample
+        par_dict = extract_parameters(self.guide_element_logs[-1], self.legend, data_line)
+        if horizontal:
+            if "end_width" in par_dict:
+                sample_width = par_dict["end_width"]
+                m_point, p_point = pos_and_rot.width_points(sample_width)
+                ax.plot([m_point[2], p_point[2]], [m_point[0], p_point[0]], color="dimgrey")
+
+        else:
+            if "end_height" in par_dict:
+                sample_height = par_dict["end_height"]
+                m_point, p_point = pos_and_rot.height_points(sample_height)
+                ax.plot([m_point[2], p_point[2]], [m_point[1], p_point[1]], color="dimgrey")
+
+        ax.set_xlabel("Distance from source [m]")
+        if horizontal:
+            ax.set_ylabel("Horizontal [m]")
+        else:
+            ax.set_ylabel("Vertical [m]")
+
+    def plot_element(self, ax, start_pr, center_function, dim_function, par_dict, n_points, horizontal, color):
+        distances = np.linspace(0, 1, n_points)
+        plus_array = np.zeros((n_points, 3))
+        minus_array = np.zeros((n_points, 3))
+
+        for index, distance in enumerate(distances):
+            this_pr = center_function(start_pr, par_dict, distance)
+            dim = dim_function(par_dict, distance, horizontal)
+            minus_array[index, :], plus_array[index, :] = this_pr.get_points(dim, horizontal)
+
+        if horizontal:
+            ax.plot(plus_array[:, 2], plus_array[:, 0], color=color)
+            ax.plot(minus_array[:, 2], minus_array[:, 0], color=color)
+        else:
+            ax.plot(plus_array[:, 2], plus_array[:, 1], color=color)
+            ax.plot(minus_array[:, 2], minus_array[:, 1], color=color)
+
+
     def plot_1D_fom_ax(self, ax, highlight_sorted_index=None, **kwargs):
         ax.scatter(range(len(self.fom)), self.fom, marker=".", label=self.scan_name, **kwargs)
         ax.plot(self.fom_sorted, color="orange")
@@ -282,10 +383,10 @@ class LogPlotter:
         self.plot_1D_cps_ax(fig.add_subplot(gs[0, 2]))
 
         geometry_h_ax = fig.add_subplot(gs[1, 0:2])
-        self.plot_guide_ax(geometry_h_ax, horizontal=True)
+        self.plot_guide_center_line_ax(geometry_h_ax, horizontal=True)
 
         geometry_v_ax = fig.add_subplot(gs[2, 0:2])
-        self.plot_guide_ax(geometry_v_ax, horizontal=False)
+        self.plot_guide_center_line_ax(geometry_v_ax, horizontal=False)
 
         par_h_ax = fig.add_subplot(gs[1, 2])
         self.plot_1D_given_type_ax(par_h_ax, type="horizontal")
