@@ -98,6 +98,11 @@ class Sample:
         # list of PlotInfo objects that will be applied to control plotting with McStasScript
         self.plot_info = PlotInfoContainer()
 
+        # Sample dimensions set during setup for each guide and retrieved from parameters object
+        self.sample_width = None
+        self.sample_height = None
+        self.sample_gap = None
+
     def __getitem__(self, item):
         """
         Ensures Sample["par_name"] returns value in parameters
@@ -130,7 +135,7 @@ class Sample:
         """
         self.parameters.lock_parameters(par1, par2)
 
-    def add_wavelength_parameters(self, instrument_parameters):
+    def add_sample_info(self, instrument_parameters):
         """
         Adds wavelength parameters to instrument_parameters
 
@@ -147,6 +152,22 @@ class Sample:
         instrument_parameters.add_parameter(min_wavelength)
         instrument_parameters.add_parameter(max_wavelength)
 
+        self.sample_width = ipars.FixedInstrumentParameter("sample_width", self["width"])
+        self.sample_height = ipars.FixedInstrumentParameter("sample_height", self["width"])
+
+        instrument_parameters.add_parameter(self.sample_width)
+        instrument_parameters.add_parameter(self.sample_height)
+
+    def get_size_parameters(self):
+        """
+        Returns sample size as parameters, only call after add_sample_info
+
+        This relies on the sample being set up already, otherwise it would
+        return information from the previous scan point.
+        """
+
+        return [self.sample_width, self.sample_height]
+
     def add_to_instrument(self, instrument, instrument_parameters):
         """
         Adds a figure of merit monitor to the McStas instrument file
@@ -164,9 +185,6 @@ class Sample:
             Parameter container where parameters can be added for optimization
         """
 
-        instrument_parameters.add_parameter(ipars.FixedInstrumentParameter("sample_width", self["width"]))
-        instrument_parameters.add_parameter(ipars.FixedInstrumentParameter("sample_height", self["height"]))
-
         mon = instrument.add_component("mon", "Divergence_monitor")
 
         mon.nh = 20
@@ -177,7 +195,7 @@ class Sample:
         mon.xwidth = "sample_width"
         mon.yheight = "sample_height"
         mon.restore_neutron = 1
-        mon.set_AT([0, 0, "sample_cp"], RELATIVE="Origin")
+        mon.set_AT([0, 0, 0], RELATIVE=self.sample_gap.end_component_name)
 
         return [self["min_wavelength"], self["max_wavelength"]]
 
@@ -211,11 +229,11 @@ class Sample:
             width_input = [0.5*width, 1.5*width]
             height_input = [0.5*height, 1.5*height]
 
-        end_gap = Element_gap.Gap(name="guide_sample_gap",
-                                  length=self["sample_guide_distance"],
-                                  start_width=width_input, start_height=height_input)
+        self.sample_gap = Element_gap.Gap(name="guide_sample_gap",
+                                       length=self["sample_guide_distance"],
+                                       start_width=width_input, start_height=height_input)
 
-        guide.add_guide_element(end_gap)
+        guide.add_guide_element(self.sample_gap)
 
     def add_analysis_to_instrument(self, instrument, instrument_parameters):
         """
@@ -236,10 +254,10 @@ class Sample:
         mon = instrument.add_component("psd_lin_horizontal", "PSDlin_monitor")
         mon.nx = 300
         mon.filename = '"psd_lin_horizontal.dat"'
-        mon.xwidth = 2*"sample_width"
+        mon.xwidth = "2.0*sample_width"
         mon.yheight = "sample_height"
         mon.restore_neutron = 1
-        mon.set_AT([0, 0, "sample_cp"], RELATIVE="Origin")
+        mon.set_AT([0, 0, 0], RELATIVE=self.sample_gap.end_component_name)
 
         info = self.plot_info.new(mon, title="Horizontal position")
         info.set_xlabel("Horizontal position [cm]")
@@ -248,7 +266,7 @@ class Sample:
         mon = instrument.add_component("psd_lin_vertical", "PSDlin_monitor")
         mon.nx = 300
         mon.filename = '"psd_lin_vertical.dat"'
-        mon.xwidth = 2*"sample_height"
+        mon.xwidth = "2.0*sample_height"
         mon.yheight = "sample_width"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
@@ -288,8 +306,8 @@ class Sample:
         mon.nx = 300
         mon.ny = 300
         mon.filename = '"psd_large.dat"'
-        mon.xwidth = 2*"sample_width"
-        mon.yheight = 2*"sample_height"
+        mon.xwidth = "2.0*sample_width"
+        mon.yheight = "2.0*sample_height"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
         mon.set_ROTATED([0, 0, 0], RELATIVE="psd_lin_horizontal")
@@ -333,7 +351,7 @@ class Sample:
         mon.ndiv = 200
         mon.filename = '"Acceptance_horizontal.dat"'
         mon.maxdiv_h = 2*self["div_horizontal"]
-        mon.xwidth = 2*"sample_width"
+        mon.xwidth = "2.0*sample_width"
         mon.yheight = "sample_height"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
@@ -349,7 +367,7 @@ class Sample:
         mon.ndiv = 200
         mon.filename = '"Acceptance_vertical.dat"'
         mon.maxdiv_h = 2 * self["div_vertical"]
-        mon.xwidth = 2 * "sample_height"
+        mon.xwidth = "2.0*sample_height"
         mon.yheight = "sample_width"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
@@ -441,7 +459,7 @@ class Sample:
         mon = instrument.add_component("psd_lin_horizontal_brill", "PSDlin_monitor")
         mon.nx = 300
         mon.filename = '"psd_lin_horizontal_brill.dat"'
-        mon.xwidth = 2 * "sample_width"
+        mon.xwidth = "2.0*sample_width"
         mon.yheight = "sample_height"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
@@ -455,7 +473,7 @@ class Sample:
         mon = instrument.add_component("psd_lin_vertical_brill", "PSDlin_monitor")
         mon.nx = 300
         mon.filename = '"psd_lin_vertical_brill.dat"'
-        mon.xwidth = 2 * "sample_height"
+        mon.xwidth = "2.0*sample_height"
         mon.yheight = "sample_width"
         mon.restore_neutron = 1
         mon.set_AT([0, 0, 1E-6], RELATIVE="PREVIOUS")
@@ -858,7 +876,13 @@ def normalize_brill_ref(data, ref_data, instrument, ref_instrument):
             monitor = instrument.get_component(data_object.name)
             ref_monitor = ref_instrument.get_component(data_object.name)
 
-            area_ratio = get_monitor_area(monitor)/get_monitor_area(ref_monitor)
+            mon_area = get_monitor_area(data_object, monitor)
+            ref_mon_area = get_monitor_area(ref_data_object, ref_monitor)
+
+            if mon_area is None or ref_mon_area is None:
+                raise RuntimeError("Area could not be read!")
+
+            area_ratio = mon_area/ref_mon_area
 
             # normalize pixel for pixel (always allowed, but less efficient)
             data_object.Intensity /= ref_data_object.Intensity*area_ratio
@@ -884,21 +908,81 @@ def write_component_dimensions(base_name, instrument):
     with open(base_name + ".yaml", 'w') as yaml_file:
         yaml.dump(component_x_y, yaml_file, default_flow_style=False)
 
-def get_monitor_area(mon):
+
+def get_monitor_area(data, component):
     """
     Gets the area of a McStas monitor that use standard xwidth / yheight input
 
     Parameters
     ----------
 
-    mon : McStasScript component object
-        McStasScript object that describe something with xwidth and yheight
+    data : McStasData object
+        data object that contains parameters used for run
+
+    component : McStasScript component object
+        component to check
     """
 
-    if hasattr(mon, "xwidth"):
-        width = mon.xwidth
-        height = mon.yheight
+    width = read_field(data, component, "xwidth")
+    height = read_field(data, component, "yheight")
 
-        return width*height  # [m^2]
+    if width is not None and height is not None:
+        return width*height # m^2
 
     return None
+
+
+def read_field(data, component, field):
+    """
+    Reads a field of a component using parameters from given dataset
+
+    data : McStasData object
+        data object that contains parameters used for run
+
+    component : McStasScript component object
+        component to check
+
+    field : str
+        name of component field to check
+    """
+
+    # If the component doesnt have the field, return
+    if not hasattr(component, field):
+        return None
+
+    # Read the input given to that field and parameters of run
+    component_field = getattr(component, field)
+    used_parameters = data.metadata.parameters
+
+    value = None
+    try:
+        # If a float or int is given directly, just read that
+        value = float(component_field)
+    except ValueError:
+        # If a parameter name is given directly, look that up
+        if component_field in used_parameters:
+            value = used_parameters[component_field]
+
+    # Last attempt to read using python eval in case a simple expression is used
+    # This will catch for example "2.0*sample_width"
+    if value is None:
+        try:
+            # Try to evaluate field with scope of all instrument parameters
+            value = eval(component_field, used_parameters)
+        except:
+            value = None
+
+    try:
+        value = float(value)
+    except ValueError:
+        # Do not want to return a string
+        value = None
+    except TypeError:
+        value = None
+
+    return value
+
+
+
+
+
