@@ -276,7 +276,7 @@ class BaseSource:
         instrument_parameters.add_parameter(dummy.end_height)
         instrument_parameters.add_parameter(dummy.start_point)
 
-        sample.add_wavelength_parameters(instrument_parameters)
+        sample.add_sample_info(instrument_parameters)
 
         # Calculate the suitable distance in this case
         # need largest divergence in each direction
